package orderservicelogic

import (
	"app-cloudep-order-server/gen_result/pb/order"
	"app-cloudep-order-server/internal/domain"
	model "app-cloudep-order-server/internal/model/mongo"
	ers "code.30cm.net/digimon/library-go/errs"
	"context"

	"app-cloudep-order-server/internal/svc"

	"github.com/zeromicro/go-zero/core/logx"
)

type ModifyOrderStatusLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewModifyOrderStatusLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ModifyOrderStatusLogic {
	return &ModifyOrderStatusLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// ModifyOrderQuery
// 0.建立訂單 1.建單失敗 2.審核中 3.付款中 4.已付款
// 5.已付款待轉帳 6.申訴中 7.交易完成
// 8.交易失敗 9.交易取消 10.交易異常 11.交易超時

type ModifyOrderQuery struct {
	BusinessID string `json:"business_id" validate:"required"`
	Status     int64  `json:"status" validate:"required,oneof=2 3 4 5 6 7 8 11"`
}

// ModifyOrderStatus 修改訂單狀態
func (l *ModifyOrderStatusLogic) ModifyOrderStatus(in *order.ModifyOrderStatusReq) (*order.OKResp, error) {
	// 驗證資料
	if err := l.svcCtx.Validate.ValidateAll(&ModifyOrderQuery{
		BusinessID: in.GetBusinessId(),
		Status:     in.GetStatus(),
	}); err != nil {
		// 錯誤代碼 06-011-00
		return nil, ers.InvalidFormat(err.Error())
	}

	_, err := l.svcCtx.OrderModel.UpdateStatus(l.ctx,
		model.UpdateStatusReq{
			BusinessID: in.GetBusinessId(),
			Status:     in.GetStatus(),
		})
	if err != nil {
		// 錯誤代碼 06-021-02
		e := domain.CommentErrorL(
			domain.ModifyOrderErrorCode,
			logx.WithContext(l.ctx),
			[]logx.LogField{
				{Key: "req", Value: in},
				{Key: "func", Value: "OrderModel.UpdateStatus"},
				{Key: "err", Value: err},
			},
			"failed to update order status:").Wrap(err)

		return nil, e
	}

	return &order.OKResp{}, nil
}
