package orderservicelogic

import (
	"app-cloudep-order-server/gen_result/pb/order"
	"app-cloudep-order-server/internal/domain"
	model "app-cloudep-order-server/internal/model/mongo"
	"context"
	"time"

	"app-cloudep-order-server/internal/svc"

	"github.com/zeromicro/go-zero/core/logx"
)

type OrderStatusTimeoutLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewOrderStatusTimeoutLogic(ctx context.Context, svcCtx *svc.ServiceContext) *OrderStatusTimeoutLogic {
	return &OrderStatusTimeoutLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// OrderStatusTimeout 訂單超時任務/cron/order-status/timeout
func (l *OrderStatusTimeoutLogic) OrderStatusTimeout(_ *order.OrderStatusTimeoutReq) (*order.OKResp, error) {
	now := time.Now().UTC().UnixNano()
	_, err := l.svcCtx.OrderModel.UpdateTimeoutOrder(l.ctx, model.UpdateTimeoutReq{
		CreateTimeBefore: now,
	})
	if err != nil {
		// 錯誤代碼 06-021-02
		e := domain.CommentErrorL(
			domain.TimeoutOrderErrorCode,
			logx.WithContext(l.ctx),
			[]logx.LogField{
				{Key: "now", Value: now},
				{Key: "func", Value: "OrderModel.UpdateTimeoutOrder"},
				{Key: "err", Value: err},
			},
			"failed to update timeout order").Wrap(err)

		return nil, e
	}

	return &order.OKResp{}, nil
}
